import pickle


def pickle_model(model, filename: str = 'model.pickle'):
    """Pickle a model using the highest protocol available."""
    with open(filename, 'wb') as f:
        pickle.dump(model, f, pickle.HIGHEST_PROTOCOL)



from sklearn.metrics import confusion_matrix, accuracy_score
import skippy as skp


def classification(dataset: str = 'digits',
                   model_type: str = 'linear_model',
                   model_name: str = 'LogisticRegression') -> str:
    data = skp.get_data(dataset)
    x_train, x_test, y_train, y_test = skp.split_data(data)
    model = skp.get_model(model_type=model_type, model_name=model_name)
    fit = model.fit(x_train, y_train)
    skp.pickle_model(model=fit, filename=f'{dataset}_{model_name}.pickle')
    predictions = fit.predict(x_test)
    confmat = confusion_matrix(y_true=y_test, y_pred=predictions)
    accuracy = accuracy_score(y_true=y_test, y_pred=predictions)
    skp.confusion_matrix_plot(cm=confmat, acc=accuracy,
                              filename=f'{dataset}_{model_name}.png')
    return (f'The {model_name} was'
            f'{accuracy:.0%} in classifying'
            f'the {dataset} dataset.')



from skippy.get_data import get_data
from skippy.split_data import split_data
from skippy.get_model import get_model
from skippy.pickle_model import pickle_model
from skippy.plot import confusion_matrix_plot, residual_plot
from skippy.classification import classification
from skippy.regression import regression

__all__ = ['get_data',
           'split_data',
           'get_model',
           'pickle_model',
           'confusion_matrix_plot',
           'residual_plot',
           'classification',
           'regression']



from importlib import import_module


def get_data(name: str):
    """load data from sklearn.datasets"""
    datasets = import_module('sklearn.datasets')
    load_dataset = getattr(datasets, 'load_'+name)
    dataset = load_dataset()
    return dataset



from sklearn.metrics import mean_squared_error, r2_score
import skippy as skp


def regression(dataset: str = 'boston',
               model_type: str = 'linear_model',
               model_name: str = 'LinearRegression'
               ) -> str:
    data = skp.get_data(dataset)
    x_train, x_test, y_train, y_test = skp.split_data(data)
    model = skp.get_model(model_type=model_type, model_name=model_name)
    fit = model.fit(x_train, y_train)
    skp.pickle_model(model=fit, filename=f'{dataset}_{model_name}.pickle')
    predictions = fit.predict(x_test)
    mse = mean_squared_error(y_true=y_test, y_pred=predictions)
    r2 = r2_score(y_true=y_test, y_pred=predictions)
    skp.residual_plot(fitted=predictions,
                      target=y_test,
                      mse=mse, r2=r2,
                      filename=f'{dataset}_{model_name}.png')
    return (f'The {model_name} explained'
            f'{r2:.0%} of the variance ($R^2$)'
            f'in the {dataset} dataset.')



import matplotlib.pyplot as plt
import seaborn as sns


def confusion_matrix_plot(cm,
                          acc=None,
                          filename='confusion_matrix.png') -> None:
    sns.heatmap(cm, square=True, annot=True, cbar=False)
    plt.xlabel('Predicted Value')
    plt.ylabel('True Value')
    if acc is not None:
        plt.title(f'Accuracy = {acc:.0%}')
    plt.savefig(filename)
    # In addition to saving the plot, you can show it
    plt.show()


def residual_plot(fitted, target,
                  mse=None, r2=None,
                  filename='residual_plot.png') -> None:
    sns.residplot(fitted, target, lowess=True, line_kws={'color': 'red'})
    plt.xlabel('Fitted Values')
    plt.ylabel('Residuals')
    if mse is not None and r2 is not None:
        plt.title(f'MSE = {mse:.0f} and $R^2$ = {r2:.0%}')
    plt.savefig(filename)
    # In addition to saving the plot, you can show it
    plt.show()



from argparse import ArgumentParser
from skippy import classification, regression


def main():
    """The main routine."""

    parser = ArgumentParser(
        description='Analyze one of the built-in scikit-learn datasets.')

    parser.add_argument('--data', '-d', default='digits',
                        help='The scikit-learn dataset name.')

    parser.add_argument('--type', '-t', default='linear_model',
                        help='The scikit-learn model type.')

    parser.add_argument('--name', '-n', default='LogisticRegression',
                        help='The scikit-learn model name.')

    args = parser.parse_args()

    kwargs = dict(dataset=args.data,
                  model_type=args.type,
                  model_name=args.name,
                  )

    if args.data in ('digits', 'iris', 'wine', 'breast_cancer'):
        classification(**kwargs)
    elif args.data in ('boston', 'diabetes'):
        regression(**kwargs)
    else:
        print("The dataset name must be digits, iris, wine, boston, breast_cancer, or diabetes.")


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="skippy",
    version="0.1.0",
    author="Martin Skarzynski",
    author_email="marskar@gmail.com",
    description="Simplified analysis of sklearn datasets",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/marskar/skippy",
    packages=['skippy'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={
        'console_scripts': [
            'skippy = skippy.__main__:main'
        ]
    },
    install_requires=[
        'scikit-learn',
        'matplotlib',
        'seaborn'
    ]
)
